from django.db import models
from django.utils.translation import ugettext_lazy as _

from dynamic_search.api import register
from inventory.models import Supplier, ItemTemplate


class PurchaseRequestStatus(models.Model):
    name = models.CharField(verbose_name=_(u'Name'), max_length=32)

    class Meta:
        verbose_name = _(u'Purchase request status')
        verbose_name_plural = _(u'Purchase request status')

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('purchase_request_state_list', [])


class PurchaseRequest(models.Model):
    user_id = models.CharField(max_length=32, null=True, blank=True, verbose_name=_(u'User defined ID'))
    issue_date = models.DateField(auto_now_add=True, verbose_name=_(u'Issue date'))
    required_date = models.DateField(null=True, blank=True, verbose_name=_(u'Date required'))
    budget = models.PositiveIntegerField(null=True, blank=True, verbose_name=_(u'Budget'))
    active = models.BooleanField(default=True, verbose_name=_(u'Active'))
    status = models.ForeignKey(PurchaseRequestStatus, null=True, blank=True, verbose_name=_(u'Status'))
    originator = models.CharField(max_length=64, null=True, blank=True, verbose_name=_(u'Originator'))
    notes = models.TextField(null=True, blank=True, verbose_name=_(u'Notes'))

    class Meta:
        verbose_name = _(u'Purchase request')
        verbose_name_plural = _(u'Purchase requests')

    def __unicode__(self):
        return '#%s (%s)' % (self.user_id if self.user_id else self.id, self.issue_date)

    @models.permalink
    def get_absolute_url(self):
        return ('purchase_request_view', [str(self.id)])


class PurchaseRequestItem(models.Model):
    purchase_request = models.ForeignKey(PurchaseRequest, related_name='items', verbose_name=_(u'Purchase request'))
    item_template = models.ForeignKey(ItemTemplate, verbose_name=_(u'Item template'))
    qty = models.PositiveIntegerField(verbose_name=_(u'Quantity'))
    notes = models.TextField(null=True, blank=True, verbose_name=_(u'Notes'))

    class Meta:
        verbose_name = _(u'Purchase request item')
        verbose_name_plural = _(u'Purchase request items')

    def __unicode__(self):
        return unicode(self.item_template)

    @models.permalink
    def get_absolute_url(self):
        return ('purchase_request_view', [str(self.purchase_request.id)])


class PurchaseOrderStatus(models.Model):
    name = models.CharField(verbose_name=_(u'Name'), max_length=32)

    class Meta:
        verbose_name = _(u'Purchase order status')
        verbose_name_plural = _(u'Purchase order status')

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('purchase_order_state_list', [])


class PurchaseOrder(models.Model):
    user_id = models.CharField(max_length=32, null=True, blank=True, verbose_name=_(u'User defined ID'))
    purchase_request = models.ForeignKey(PurchaseRequest, null=True, blank=True, verbose_name=_(u'Purchase request'))
    supplier = models.ForeignKey(Supplier, related_name='purchase_orders', verbose_name=_(u'Supplier'))
    issue_date = models.DateField(auto_now_add=True, verbose_name=_(u'Issue date'))
    required_date = models.DateField(null=True, blank=True, verbose_name=_(u'Date required'))
    active = models.BooleanField(default=True, verbose_name=_(u'Active'))
    notes = models.TextField(null=True, blank=True, verbose_name=_(u'Notes'))
    status = models.ForeignKey(PurchaseOrderStatus, null=True, blank=True, verbose_name=_(u'Status'))

    class Meta:
        verbose_name = _(u'Purchase order')
        verbose_name_plural = _(u'Purchase orders')

    def __unicode__(self):
        return '#%s (%s)' % (self.user_id if self.user_id else self.id, self.issue_date)

    @models.permalink
    def get_absolute_url(self):
        return ('purchase_order_view', [str(self.id)])


class PurchaseOrderItemStatus(models.Model):
    name = models.CharField(verbose_name=_(u'Name'), max_length=32)

    class Meta:
        verbose_name = _(u'Purchase order item status')
        verbose_name_plural = _(u'Purchase order item status')

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('purchase_order_item_state_list', [])


class PurchaseOrderItem(models.Model):
    purchase_order = models.ForeignKey(PurchaseOrder, related_name='items', verbose_name=_(u'Purchase order'))
    item_template = models.ForeignKey(ItemTemplate, verbose_name=_(u'Item template'))
    agreed_price = models.PositiveIntegerField(null=True, blank=True, verbose_name=_(u'Agreed price'))
    active = models.BooleanField(default=True, verbose_name=_(u'Active'))
    status = models.ForeignKey(PurchaseOrderItemStatus, null=True, blank=True, verbose_name=_(u'Status'))
    qty = models.PositiveIntegerField(verbose_name=_(u'Quantity'))
    received_qty = models.PositiveIntegerField(default=0, null=True, blank=True, verbose_name=_(u'received'))

    class Meta:
        verbose_name = _(u'Purchase order item')
        verbose_name_plural = _(u'Purchase order items')

    def __unicode__(self):
        return unicode(self.item_template)

    @models.permalink
    def get_absolute_url(self):
        return ('purchase_order_view', [str(self.purchase_order.id)])


register(PurchaseRequestStatus, _(u'Purchase request status'), ['name'])
register(PurchaseRequest, _(u'Purchase request'), ['user_id', 'id', 'budget', 'required_date', 'status__name', 'originator'])
register(PurchaseRequestItem, _(u'Purchase request item'), ['item_template__description', 'qty', 'notes'])
register(PurchaseOrderStatus, _(u'Purchase order status'), ['name'])
register(PurchaseOrderItemStatus, _(u'Purchase order item status'), ['name'])
register(PurchaseOrder, _(u'Purchase order'), ['user_id', 'id', 'required_date', 'status__name', 'supplier__name', 'notes'])
register(PurchaseOrderItem, _(u'Purchase order item'), ['item_template__description', 'qty'])



from __future__ import absolute_import

import re

from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.db.models import Q

from common.utils import encapsulate

from .api import search_list
from .forms import SearchForm

#original code from:
#http://www.julienphalip.com/blog/2008/08/16/adding-search-django-site-snap/

def normalize_query(query_string,
                    findterms=re.compile(r'"([^"]+)"|(\S+)').findall,
                    normspace=re.compile(r'\s{2,}').sub):
    """ Splits the query string in invidual keywords, getting rid of unecessary spaces
        and grouping quoted words together.
        Example:

        >>> normalize_query('  some random  words "with   quotes  " and   spaces')
        ['some', 'random', 'words', 'with quotes', 'and', 'spaces']

    """
    return [normspace(' ', (t[0] or t[1]).strip()) for t in findterms(query_string)]


def get_query(terms, search_fields):
    """ Returns a query, that is a combination of Q objects. That combination
        aims to search keywords within a model by testing the given search fields.

    """
    query = None # Query to search for every search term
    #terms = normalize_query(query_string)
    for term in terms:
        or_query = None # Query to search for a given term in each field
        for field_name in search_fields:
            q = Q(**{'%s__icontains' % field_name: term})
            if or_query is None:
                or_query = q
            else:
                or_query = or_query | q
        if query is None:
            query = or_query
        else:
            query = query & or_query
    return query


def search(request):
    query_string = ''
    found_entries = {}
    object_list = []

    if ('q' in request.GET) and request.GET['q'].strip():
        query_string = request.GET['q']
        form = SearchForm(initial={'q': query_string})

        terms = normalize_query(query_string)

        for model, data in search_list.items():
            query = get_query(terms, data['fields'])

            results = model.objects.filter(query)
            if results:
                found_entries[data['text']] = results
                for result in results:
                    object_list.append(result)
    else:
        form = SearchForm()

    return render_to_response('dynamic_search/search_results.html', {
                            'query_string': query_string,
                            'found_entries': found_entries,
                            'form': form,
                            'object_list': object_list,
                            'form_title': _(u'Search'),
                            'extra_columns': [{'name':_(u'type'), 'attribute': encapsulate(lambda x: x._meta.verbose_name[0].upper() + x._meta.verbose_name[1:])}],
                            'title': _(u'results with: %s') % query_string
                            },
                          context_instance=RequestContext(request))



from django.template.defaultfilters import stringfilter
from django.template import Library, Node, Variable
from django.core.urlresolvers import reverse

from assets.models import State

register = Library()


class GetAllStatesNode(Node):
    def __init__(self, variable):
        self.variable = variable

    def render(self, context):
        context[self.variable] = State.objects.all()
        return ''


@register.tag
def get_all_states(parser, token):
    args = token.contents.split()
    if len(args) != 3 or args[1] != 'as':
        raise TemplateSyntaxError("'get_all_states' requires 'as variable' (got %r)" % args)
    return GetAllStatesNode(args[2])



from django import forms 
from django.template import TemplateSyntaxError, Library, \
                            VariableDoesNotExist, Node, Variable
from django.conf import settings

register = Library()


class StylingNode(Node):
    def __init__(self, form_name, *args, **kwargs):
        self.form_name = form_name
        
    def render(self, context):
        form = Variable(self.form_name).resolve(context)
        for field_name, field in form.fields.items():

            if isinstance(field.widget, forms.widgets.TextInput):
                field.widget.attrs['class'] = 'text_field'
            elif isinstance(field.widget, forms.widgets.PasswordInput):
                field.widget.attrs['class'] = 'text_field'
            elif isinstance(field.widget, forms.widgets.Textarea):
                field.widget.attrs['class'] = 'text_area'                

        context[self.form_name] = form
        return ''                



@register.tag
def add_classes_to_form(parser, token):
    args = token.split_contents()
    return StylingNode(args[1])



import copy

object_navigation = {}
menu_links = []


def register_links(src, links, menu_name=None):
    if menu_name in object_navigation:
        if hasattr(src, '__iter__'):
            for one_src in src:
                if one_src in object_navigation[menu_name]:
                    object_navigation[menu_name][one_src]['links'].extend(links)
                else:
                    object_navigation[menu_name][one_src] = {'links': copy.copy(links)}
        else:
            if src in object_navigation[menu_name]:
                object_navigation[menu_name][src]['links'].extend(links)
            else:
                object_navigation[menu_name][src] = {'links': links}
    else:
        object_navigation[menu_name] = {}
        if hasattr(src, '__iter__'):
            for one_src in src:
                object_navigation[menu_name][one_src] = {'links': links}
        else:
            object_navigation[menu_name] = {src: {'links': links}}


def register_menu(links):
    for link in links:
        menu_links.append(link)

    menu_links.sort(lambda x, y: 1 if x > y else -1, lambda x: x['position'] if 'position' in x else 1)
